local M = {}

M.modeMap = {
	n = "NORMAL", i = "INSERT", R = "REPLACE", v = "VISUAL", V = "V-LINE", [''] = "V-BLOCK",
	c = "COMMAND", s = "SELECT", S = "S-LINE", [''] = "S-BLOCK", nt = "NORMAL", t = "TERMINAL"
}

GetMode = function()
	local m = vim.api.nvim_get_mode().mode
	if M.modeMap[m] ~= nil then
		return Highlight(" "..M.modeMap[m].." ","StatusLineMode")
	else
		return ""
	end
end

GetDate = function()
	return tostring(vim.fn.strftime('%Y-%m-%d'))
end

GetTime = function()
	local time = tostring(vim.fn.strftime('%H:%M:%S'))
	return time
end

GetLSP = function()
	local clients = vim.lsp.get_clients({bufnr = 0})
	if next(clients) ~= nil then
		local name = string.format('%s',clients[1].name)
		local root = clients[1].root_dir
		if name == 'matlab_ls' and root ~= nil then
			local sandbox = root:match(".+/([^/]+)$")
			return string.format(' {◉%s} [%s] ',name,sandbox)
		else
			return string.format(' {◉%s} ',name)
		end
	else
		return ' {◉} '
	end
end

M.diagMap = {
	err =  vim.diagnostic.severity.ERROR,
	warn = vim.diagnostic.severity.WARNING,
	info = vim.diagnostic.severity.INFORMATION,
	hint = vim.diagnostic.severity.HINT
}

M.diagSymMap = {
	err =  "✖",
	warn = "⚠",
	info = "●",
	hint = "✦"
}

GetDiag = function(type)
	return string.format("%s%d ",M.diagSymMap[type],#vim.diagnostic.get(0,{severity=M.diagMap[type]}))
end

GetGitStatus = function()
	local signs = vim.b.gitsigns_status_dict or {head = '', added = 0, changed = 0, removed = 0}
	local inRepo = signs.head ~= ''

	if inRepo and signs.added ~= nil then
		return string.format(
	    '%s%s%s%s',
    	Highlight(" {↣"..signs.head.."}","StatusLineGit"),
		Highlight(" +"..signs.added,"StatusLineGitAdd"),
		Highlight(" ~"..signs.changed,"StatusLineGitChange"),
		Highlight(" -"..signs.removed.." ","StatusLineGitDelete")
		)
	elseif inRepo and signs.added == nil then
		return string.format(
	    '%s%s%s%s',
    	Highlight(" {↣"..signs.head.." -- untracked}","StatusLineGit"),
		Highlight(" +0","StatusLineGitAdd"),
		Highlight(" ~0","StatusLineGitChange"),
		Highlight(" -0 ","StatusLineGitDelete")
		)
	else
		return string.format(
	    '%s%s%s%s',
    	Highlight(" {↣}","StatusLineGit"),
		Highlight(" +0","StatusLineGitAdd"),
		Highlight(" ~0","StatusLineGitChange"),
		Highlight(" -0 ","StatusLineGitDelete")
		)
	end

end

Highlight = function(inp,hi)
	return "%#" .. hi .. "#" .. inp .. "%*"
end

function M.statusline()
	local sline = ""
	sline = sline .. GetMode()
				  .. Highlight(" %<%f ","StatusLineFile") .. " %y%m%r%="
				  .. "%-5.(%l:%c%V%) %p%% "
	              .. GetGitStatus()
				  .. Highlight(GetLSP(),"StatusLineLSP")
				  .. Highlight(GetDiag("err"),"StatusLineDiagnosticError")
				  .. Highlight(GetDiag("warn"),"StatusLineDiagnosticWarn")
				  .. Highlight(GetDiag("info"),"StatusLineDiagnosticInfo")
				  .. Highlight(GetDiag("hint"),"StatusLineDiagnosticHint")
	return sline
end

function M.winbar()
	return "%< %f %y%h%m%r%=" .. GetTime()
end

--local timer = vim.uv.new_timer()
--timer:start(0, 5000, vim.schedule_wrap(function()
--    vim.cmd("let &stl=&stl")
--end))

return M
